# Public generated API
from grpc_helper.api.config_pb2 import *  # NOQA: F401,F403
from grpc_helper.api.server_pb2 import *  # NOQA: F401,F403
from grpc_helper.api.common_pb2 import *  # NOQA: F401,F403
from grpc_helper.api.events_pb2 import *  # NOQA: F401,F403
from grpc_helper.api.logger_pb2 import *  # NOQA: F401,F403



from configparser import ConfigParser
from pathlib import Path

from pkg_resources import DistributionNotFound, get_distribution

__title__ = "grpc-helper"
try:
    __version__ = get_distribution(__title__).version
except DistributionNotFound:  # pragma: no cover
    # For debug
    with (Path(__file__).parent.parent.parent.parent / "setup.cfg").open("r") as f:
        c = ConfigParser()
        c.read_file(f.readlines())
        __version__ = c.get("metadata", "version")

# Public RPC API
from grpc_helper.cli import RpcCliParser
from grpc_helper.client import RpcClient
from grpc_helper.errors import RpcException
from grpc_helper.folders import Folders
from grpc_helper.manager import RpcManager, RpcProxiedManager
from grpc_helper.server import RpcServer, RpcServiceDescriptor
from grpc_helper.static_config import RpcStaticConfig

__all__ = ["RpcServer", "RpcServiceDescriptor", "RpcClient", "RpcException", "RpcManager", "RpcProxiedManager", "Folders", "RpcCliParser", "RpcStaticConfig"]



import copy
import time
import traceback
from logging import getLogger
from threading import current_thread

from grpc_helper.api import Result, ResultCode, ResultStatus, ServiceInfo
from grpc_helper.client import RpcClient
from grpc_helper.errors import RpcException
from grpc_helper.manager import RpcManager
from grpc_helper.meta import RpcMetadata
from grpc_helper.static_config import RpcStaticConfig
from grpc_helper.utils import trace_rpc


class RpcServerMethod:
    def __init__(self, name: str, manager: RpcManager, stub: object, return_type: object, info: ServiceInfo, server: object):
        self.manager_method = getattr(manager, name) if manager is not None else None
        self.name = name
        self.logger = manager.logger if manager is not None else getLogger("RpcServer")
        self.return_type = return_type
        self.info = info
        self.server = server
        self.stub = stub

    def prelude(self, request, context) -> str:
        # Remember call for debug dump
        input_trace = trace_rpc(True, request, context=context)
        logged_call = f"Thread 0x{current_thread().ident:016x} -- " + input_trace
        with self.server.lock:
            self.server.calls.append(logged_call)
        self.logger.debug(input_trace)
        return logged_call

    def epilog(self, logged_call):
        # Forget call from debug dump
        with self.server.lock:
            self.server.calls.remove(logged_call)

    def report_exception(self, context, e: Exception):
        # Something happened during the RPC execution
        stack = "".join(traceback.format_tb(e.__traceback__))
        self.logger.error(f"Exception occurred: {e}\n{stack}")

        # Extract RC if this was a known error
        rc = e.rc if isinstance(e, RpcException) else ResultCode.ERROR

        # Build result according to return type
        r = Result(code=rc, msg=str(e), stack=stack)
        return ResultStatus(r=r) if self.return_type is None else self.return_type(r=r)

    def delegate_call(self, request, context):
        # Verify API version
        metadata = RpcMetadata.from_context(context)
        client_version = None
        if len(metadata.api_version):
            client_version = int(metadata.api_version)
            if client_version > self.info.current_api_version:
                raise RpcException(
                    f"Server current API version ({self.info.current_api_version}) is too old for client API version ({client_version})",
                    rc=ResultCode.ERROR_API_SERVER_TOO_OLD,
                )
            elif client_version < self.info.supported_api_version:
                raise RpcException(
                    f"Client API version ({client_version}) is too old for server supported API version ({self.info.current_api_version})",
                    rc=ResultCode.ERROR_API_CLIENT_TOO_OLD,
                )

        # Proxy?
        if self.info.is_proxy:
            # Delegate to remote proxy, if port is set
            first_try = time.time()
            while self.info.proxy_port == 0:
                # Wait a bit more for proxy registration
                if (time.time() - first_try) < RpcStaticConfig.CLIENT_TIMEOUT.float_val:
                    # Server is not available, and timeout didn't expired yet: sleep and retry
                    time.sleep(0.5)
                    self.logger.debug(f"Proxy not registered yet for method {self.name}: wait a bit...")
                else:
                    # Timeout expired... notify the caller
                    raise RpcException("Proxy didn't registered in time for method call", ResultCode.ERROR_PROXY_UNREGISTERED)

            # Reuse metadata from context
            client_meta = copy.copy(metadata)
            client_meta.client = f"{client_meta.client}(proxied)"

            # Temporary client to proxied server (that won't raise exceptions: let's simply forward the output message to final client)
            client = RpcClient(
                self.info.proxy_host if len(self.info.proxy_host) else RpcStaticConfig.MAIN_HOST.str_val,
                self.info.proxy_port,
                {"stub": (self.stub, client_version if client_version is not None else self.info.current_api_version)},
                name=client_meta,
                timeout=RpcStaticConfig.CLIENT_TIMEOUT.float_val,
                logger=self.logger,
                exception=False,
            )

            # Call remote method
            result_provider = getattr(client.stub, self.name)(request)
        else:
            # Not a proxy method, delegate to manager
            result_provider = self.manager_method(request)

        return result_provider


class RpcSimpleMethod(RpcServerMethod):
    def __call__(self, request, context):
        # Call prelude
        logged_call = self.prelude(request, context)

        try:
            # Delegate call (simple output)
            result = self.delegate_call(request, context)
        except Exception as e:
            # Handle exception
            result = self.report_exception(context, e)

        # Call epilog
        self.epilog(logged_call)
        self.logger.debug(trace_rpc(False, result, context=context))

        return result


class RpcStreamingMethod(RpcServerMethod):
    def __call__(self, request, context):
        # Call prelude
        logged_call = self.prelude(request, context)

        try:
            # Delegate call (streaming output)
            result_provider = self.delegate_call(request, context)
            for result in result_provider:
                self.logger.debug(trace_rpc(False, result, context=context))
                yield result
        except Exception as e:
            # Handle exception
            result = self.report_exception(context, e)
            self.logger.debug(trace_rpc(False, result, context=context))
            yield result

        # Call epilog
        self.epilog(logged_call)



from grpc_helper.api import ResultCode


class RpcException(Exception):  # NOQA: N818
    """
    Common error class, holding an error code (typically to be returned in RPC response messages)
    """

    def __init__(self, message: str, rc: int = ResultCode.ERROR):
        super().__init__(message)
        self.rc = rc



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_helper/api/common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grpc_helper/api/common.proto",
    package="",
    syntax="proto3",
    serialized_options=b"\n\017grpc_helper.apiP\001",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cgrpc_helper/api/common.proto"?\n\x06Result\x12\x19\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x0b.ResultCode\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\r\n\x05stack\x18\x03 \x01(\t""\n\x0cResultStatus\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result"\x07\n\x05\x45mpty"/\n\x06\x46ilter\x12\r\n\x05names\x18\x01 \x03(\t\x12\x16\n\x0eignore_unknown\x18\x02 \x01(\x08*\xc0\x04\n\nResultCode\x12\x06\n\x02OK\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x08\n\x04\x41RGS\x10\x02\x12\x13\n\x0f\x45RROR_PORT_BUSY\x10\n\x12\x1c\n\x18\x45RROR_API_CLIENT_TOO_OLD\x10\x0b\x12\x1c\n\x18\x45RROR_API_SERVER_TOO_OLD\x10\x0c\x12\r\n\tERROR_RPC\x10\r\x12\x1c\n\x18\x45RROR_PROXY_UNREGISTERED\x10\x0e\x12\x19\n\x15\x45RROR_STREAM_SHUTDOWN\x10\x0f\x12\x17\n\x13\x45RROR_PARAM_MISSING\x10\x14\x12\x17\n\x13\x45RROR_PARAM_INVALID\x10\x15\x12\x1b\n\x17\x45RROR_PARAM_UNSUPPORTED\x10\x16\x12\x16\n\x12\x45RROR_ITEM_UNKNOWN\x10\x1e\x12\x17\n\x13\x45RROR_ITEM_CONFLICT\x10\x1f\x12\x13\n\x0f\x45RROR_ITEM_BUSY\x10 \x12\x17\n\x13\x45RROR_ITEM_MULTIPLE\x10!\x12\x17\n\x13\x45RROR_MODEL_INVALID\x10(\x12\x16\n\x12\x45RROR_MODEL_SYNTAX\x10)\x12\x18\n\x14\x45RROR_REQUEST_FAILED\x10\x32\x12\x19\n\x15\x45RROR_REQUEST_TIMEOUT\x10\x33\x12\x1d\n\x19\x45RROR_REQUEST_UNSUPPORTED\x10\x34\x12\x1b\n\x17\x45RROR_STATE_UNREACHABLE\x10<\x12\x1a\n\x16\x45RROR_STATE_UNEXPECTED\x10=\x12\x10\n\x0c\x45RROR_CUSTOM\x10\x64\x42\x13\n\x0fgrpc_helper.apiP\x01\x62\x06proto3',
)

_RESULTCODE = _descriptor.EnumDescriptor(
    name="ResultCode",
    full_name="ResultCode",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(name="OK", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(name="ERROR", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(name="ARGS", index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name="ERROR_PORT_BUSY", index=3, number=10, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_API_CLIENT_TOO_OLD", index=4, number=11, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_API_SERVER_TOO_OLD", index=5, number=12, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(name="ERROR_RPC", index=6, number=13, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name="ERROR_PROXY_UNREGISTERED", index=7, number=14, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_STREAM_SHUTDOWN", index=8, number=15, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_PARAM_MISSING", index=9, number=20, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_PARAM_INVALID", index=10, number=21, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_PARAM_UNSUPPORTED", index=11, number=22, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_ITEM_UNKNOWN", index=12, number=30, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_ITEM_CONFLICT", index=13, number=31, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_ITEM_BUSY", index=14, number=32, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_ITEM_MULTIPLE", index=15, number=33, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_MODEL_INVALID", index=16, number=40, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_MODEL_SYNTAX", index=17, number=41, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_REQUEST_FAILED", index=18, number=50, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_REQUEST_TIMEOUT", index=19, number=51, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_REQUEST_UNSUPPORTED", index=20, number=52, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_STATE_UNREACHABLE", index=21, number=60, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_STATE_UNEXPECTED", index=22, number=61, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR_CUSTOM", index=23, number=100, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=192,
    serialized_end=768,
)
_sym_db.RegisterEnumDescriptor(_RESULTCODE)

ResultCode = enum_type_wrapper.EnumTypeWrapper(_RESULTCODE)
OK = 0
ERROR = 1
ARGS = 2
ERROR_PORT_BUSY = 10
ERROR_API_CLIENT_TOO_OLD = 11
ERROR_API_SERVER_TOO_OLD = 12
ERROR_RPC = 13
ERROR_PROXY_UNREGISTERED = 14
ERROR_STREAM_SHUTDOWN = 15
ERROR_PARAM_MISSING = 20
ERROR_PARAM_INVALID = 21
ERROR_PARAM_UNSUPPORTED = 22
ERROR_ITEM_UNKNOWN = 30
ERROR_ITEM_CONFLICT = 31
ERROR_ITEM_BUSY = 32
ERROR_ITEM_MULTIPLE = 33
ERROR_MODEL_INVALID = 40
ERROR_MODEL_SYNTAX = 41
ERROR_REQUEST_FAILED = 50
ERROR_REQUEST_TIMEOUT = 51
ERROR_REQUEST_UNSUPPORTED = 52
ERROR_STATE_UNREACHABLE = 60
ERROR_STATE_UNEXPECTED = 61
ERROR_CUSTOM = 100


_RESULT = _descriptor.Descriptor(
    name="Result",
    full_name="Result",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="code",
            full_name="Result.code",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="msg",
            full_name="Result.msg",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="stack",
            full_name="Result.stack",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=32,
    serialized_end=95,
)


_RESULTSTATUS = _descriptor.Descriptor(
    name="ResultStatus",
    full_name="ResultStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="ResultStatus.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=97,
    serialized_end=131,
)


_EMPTY = _descriptor.Descriptor(
    name="Empty",
    full_name="Empty",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=133,
    serialized_end=140,
)


_FILTER = _descriptor.Descriptor(
    name="Filter",
    full_name="Filter",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="names",
            full_name="Filter.names",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ignore_unknown",
            full_name="Filter.ignore_unknown",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=142,
    serialized_end=189,
)

_RESULT.fields_by_name["code"].enum_type = _RESULTCODE
_RESULTSTATUS.fields_by_name["r"].message_type = _RESULT
DESCRIPTOR.message_types_by_name["Result"] = _RESULT
DESCRIPTOR.message_types_by_name["ResultStatus"] = _RESULTSTATUS
DESCRIPTOR.message_types_by_name["Empty"] = _EMPTY
DESCRIPTOR.message_types_by_name["Filter"] = _FILTER
DESCRIPTOR.enum_types_by_name["ResultCode"] = _RESULTCODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Result = _reflection.GeneratedProtocolMessageType(
    "Result",
    (_message.Message,),
    {
        "DESCRIPTOR": _RESULT,
        "__module__": "grpc_helper.api.common_pb2"
        # @@protoc_insertion_point(class_scope:Result)
    },
)
_sym_db.RegisterMessage(Result)

ResultStatus = _reflection.GeneratedProtocolMessageType(
    "ResultStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _RESULTSTATUS,
        "__module__": "grpc_helper.api.common_pb2"
        # @@protoc_insertion_point(class_scope:ResultStatus)
    },
)
_sym_db.RegisterMessage(ResultStatus)

Empty = _reflection.GeneratedProtocolMessageType(
    "Empty",
    (_message.Message,),
    {
        "DESCRIPTOR": _EMPTY,
        "__module__": "grpc_helper.api.common_pb2"
        # @@protoc_insertion_point(class_scope:Empty)
    },
)
_sym_db.RegisterMessage(Empty)

Filter = _reflection.GeneratedProtocolMessageType(
    "Filter",
    (_message.Message,),
    {
        "DESCRIPTOR": _FILTER,
        "__module__": "grpc_helper.api.common_pb2"
        # @@protoc_insertion_point(class_scope:Filter)
    },
)
_sym_db.RegisterMessage(Filter)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import os
from pathlib import Path
from typing import Dict, List

from grpc_helper.api import ConfigApiVersion, ConfigItem, ConfigStatus, ConfigUpdate, Filter, ResultCode
from grpc_helper.api.config_pb2_grpc import ConfigServiceServicer, ConfigServiceStub
from grpc_helper.client import RpcClient
from grpc_helper.config.cfg_item import Config
from grpc_helper.errors import RpcException
from grpc_helper.folders import Folders
from grpc_helper.manager import RpcManager

# Config file name
CONFIG_FILE = "config.json"


class ConfigManager(ConfigServiceServicer, RpcManager):
    """
    Configuration manager, holding static/user config items, and implementing the ConfigService API
    """

    def __init__(self, folders: Folders = None, cli_config: Dict[str, str] = None, static_items: List[Config] = None, user_items: List[Config] = None):
        RpcManager.__init__(self, CONFIG_FILE, self.__validate_config_file)
        self.folders = folders if folders is not None else Folders()
        self.static_items = self.__serialize_items(static_items)
        self.user_items = self.__serialize_items(user_items)
        self.cli_config = cli_config if cli_config is not None else {}

        # Can't support an item in both lists
        conflicting_items = list(set(self.static_items.keys()) & set(self.user_items.keys()))
        if len(conflicting_items):
            raise RpcException("Some config items defined as both static and user ones: " + ", ".join(conflicting_items), ResultCode.ERROR_MODEL_INVALID)

        # Prepare default/current values dict
        defaults = self.__load_defaults()
        currents = self._load_config(self.folders.workspace)

        # Load default/current values for all items
        for name, item in self.__all_items.items():
            # Validate and set default value
            default_val = defaults[name]
            item.validate(name, default_val)
            item.item.default_value = default_val

            # Validate persisted current (validator may have changed)
            if name in currents:
                current_val = currents[name]
                try:
                    item.update(current_val)
                    continue
                except Exception:
                    self.logger.warning(f"Can't load invalid persisted value '{current_val}' for config item {name}; use default one")

            # Default (no persisted current value or validation error on persisted value)
            item.update(default_val)

    def _load(self):
        # Simple dump of all loaded items
        self.logger.info("Items dump on load:")
        for item_type, name, item in [
            (item_type, name, item) for item_type, item_map in [("static", self.static_items), ("user", self.user_items)] for name, item in item_map.items()
        ]:
            self.logger.info(f" - [{item_type}] {name}: {item.str_val} (default: {item.default_value})")

    def __serialize_items(self, input_list: list) -> Dict[str, Config]:
        # Browse input list items, that may be:
        # - either a Config instance
        # - or a ConfigHolder containing a list of Config instances
        out = {}
        if input_list is not None:
            for candidate in input_list:
                if isinstance(candidate, Config):
                    # Simply add to output
                    out[candidate.name] = candidate
                else:
                    # Assume this is a ConfigHolder class: serialize all items from the holder
                    out.update({n: i for n, i in map(lambda i: (i.name, i), candidate.all_config_items())})
        return out

    @property
    def __all_items(self) -> Dict[str, Config]:
        out = dict(self.static_items)
        out.update(self.user_items)
        return out

    def __validate_config_file(self, config_file: Path, json_model):
        if not isinstance(json_model, dict) or any(not isinstance(v, str) for v in json_model.values()):
            raise RpcException(f"Invalid config json file (expecting a simple str:str object): {config_file}", ResultCode.ERROR_MODEL_INVALID)

    def __load_env_config(self) -> Dict[str, str]:
        # Check if configuration item default value is provided by environment
        env_defaults = {}
        for name in self.__all_items.keys():
            # Transform to env var name --> env var for foo_bar_12 config name is FOO_BAR_12
            env_name = name.upper().replace("-", "_")
            if env_name in os.environ:
                env_defaults[name] = os.environ[env_name]
        return env_defaults

    def __load_defaults(self) -> Dict[str, str]:
        # Layer 1: hard-coded values
        defaults = {i.name: i.hard_coded_default_value for i in self.__all_items.values()}
        self.logger.debug(f"Loading defaults (hard-coded): {defaults}")

        # Layer 2: system shared config file
        defaults.update(self._load_config(self.folders.system))
        self.logger.debug(f"Loading defaults (from system config at {self.folders.system}): {defaults}")

        # Layer 3: user config file
        defaults.update(self._load_config(self.folders.user))
        self.logger.debug(f"Loading defaults (from user config at {self.folders.user}): {defaults}")

        # Layer 4: environment
        defaults.update(self.__load_env_config())
        self.logger.debug(f"Loading defaults (from environment): {defaults}")

        # Layer 5: command-line
        defaults.update(self.cli_config)
        self.logger.debug(f"Loading defaults (from cli options): {defaults}")

        return defaults

    def __persist(self):
        # Persist non-default public values
        self._save_config({i.name: i.str_val for i in filter(lambda i: i.str_val != i.default_value, self.user_items.values())})

    def __check_items(self, input_names: list, items_to_check: dict, ignore_unknown: bool, empty_ok: bool = False):
        # Check for empty list
        if not empty_ok and len(input_names) == 0:
            raise RpcException("Input request list is empty", ResultCode.ERROR_PARAM_MISSING)

        # Check for empty name in list
        if any(n == "" for n in input_names):
            raise RpcException("At least one empty name found in input request", ResultCode.ERROR_PARAM_MISSING)

        # Check filter for unknown items
        unknown_items = list(filter(lambda n: n not in items_to_check.keys(), input_names))
        if not ignore_unknown and len(unknown_items):
            raise RpcException("Unknown config item names in filter request: " + ", ".join(unknown_items), ResultCode.ERROR_ITEM_UNKNOWN)

    def __filter_items(self, names: List[str]) -> List[ConfigItem]:
        return list(map(lambda x: self.user_items[x].item, filter(lambda x: x in self.user_items, names if len(names) else self.user_items.keys())))

    def __merged_items(self, names: List[str], check_conflicts: bool = False) -> Dict[str, ConfigItem]:
        # Delegate to all proxied servers + merge with local items
        merged_items = {}
        dump_all_filter = Filter(names=names, ignore_unknown=True)

        # Dump items from proxied clients
        items_dumps = [self.__filter_items(names)]
        for client in self.__proxied_config_clients:
            self.logger.debug(f"Dump items from remote ({client.target_host})")
            items_dumps.append(client.config.get(dump_all_filter).items)

        # Iterate on dumps
        for items_dump in items_dumps:
            # Merge proxied items
            for item in items_dump:
                if item.name in merged_items:
                    if item.value != merged_items[item.name].value and check_conflicts:
                        # Conflict between services; needs to be raised as an error
                        raise RpcException(
                            f"Proxied values conflict for config item {item.name}: {item.value} != {merged_items[item.name].value}",
                            rc=ResultCode.ERROR_ITEM_CONFLICT,
                        )
                    else:
                        self.logger.debug(f"Item {item.name} (value: {item.value}) already merged; keep previous value ({merged_items[item.name].value})")
                else:
                    # Grab this proxied item
                    merged_items[item.name] = item
                    self.logger.debug(f"Item {item.name} merged (value: {item.value})")
        return merged_items

    def get(self, request: Filter) -> ConfigStatus:
        """
        Get configuration items, according to input filter
        """

        with self.lock:
            # Basic checks
            merged_items = self.__merged_items(request.names, True)
            self.__check_items(request.names, merged_items, request.ignore_unknown, True)

            # Return filtered list
            return ConfigStatus(items=merged_items.values())

    def reset(self, request: Filter) -> ConfigStatus:
        """
        Reset configuration items, according to input filter
        """

        with self.lock:
            # Basic checks
            merged_items = self.__merged_items(request.names, False)
            self.__check_items(request.names, merged_items, request.ignore_unknown)

            # Delegate to proxied servers
            for client in self.__proxied_config_clients:
                self.logger.debug(f"Reset items on remote ({client.target_host})")
                client.config.reset(Filter(names=request.names, ignore_unknown=True))

            # Reset all local items to their default values
            for name in filter(lambda n: n in self.user_items, request.names):
                self.user_items[name].reset()

            # Get again to build returned values
            merged_items = self.__merged_items(request.names, True)
            return ConfigStatus(items=merged_items.values())

    def set(self, request: ConfigUpdate) -> ConfigStatus:  # NOQA:A003
        """
        Update configuration items, according to input request
        """

        with self.lock:
            # Basic checks
            req_map = {r.name: r.value for r in request.items}
            merged_items = self.__merged_items(req_map.keys(), False)
            self.__check_items(req_map.keys(), merged_items, request.ignore_unknown)

            # Validate local items new values
            for name, value in filter(lambda tpl: tpl[0] in self.user_items, req_map.items()):
                self.user_items[name].validate(name, value)

            # Delegate to proxied servers
            for client in self.__proxied_config_clients:
                self.logger.debug(f"Set items on remote ({client.target_host})")
                client.config.set(ConfigUpdate(items=list(request.items), ignore_unknown=True))

            # Finally, update local values
            for name, value in filter(lambda tpl: tpl[0] in self.user_items, req_map.items()):
                self.user_items[name].update(value)

            # Persist updated local values
            self.__persist()

            # Get again to build returned values
            merged_items = self.__merged_items(req_map.keys(), True)
            return ConfigStatus(items=merged_items.values())

    @property
    def __proxied_config_clients(self) -> List[RpcClient]:
        return self._proxied_clients({"config": (ConfigServiceStub, ConfigApiVersion.CONFIG_API_CURRENT)})



import faulthandler
import inspect
import logging
import os
import signal
import time
import traceback
from concurrent import futures
from dataclasses import dataclass
from threading import Event, Thread
from types import ModuleType
from typing import Callable, Dict, List, NoReturn, TypeVar, Union

from google.protobuf.internal.enum_type_wrapper import EnumTypeWrapper
from grpc import Server, insecure_channel, server

import grpc_helper
from grpc_helper.api import ConfigApiVersion
from grpc_helper.api import Event as RpcEvent
from grpc_helper.api import (
    EventApiVersion,
    EventProperty,
    Filter,
    LoggerApiVersion,
    MultiServiceInfo,
    ProxyRegisterRequest,
    Result,
    ResultCode,
    ResultStatus,
    ServerApiVersion,
    ServiceInfo,
    ShutdownRequest,
)
from grpc_helper.api.config_pb2_grpc import ConfigServiceStub, add_ConfigServiceServicer_to_server
from grpc_helper.api.events_pb2_grpc import EventServiceStub, add_EventServiceServicer_to_server
from grpc_helper.api.logger_pb2_grpc import LoggerServiceStub, add_LoggerServiceServicer_to_server
from grpc_helper.api.server_pb2_grpc import RpcServerServiceServicer, RpcServerServiceStub, add_RpcServerServiceServicer_to_server
from grpc_helper.client import RpcClient
from grpc_helper.config.cfg_item import Config
from grpc_helper.config.cfg_manager import ConfigManager
from grpc_helper.errors import RpcException
from grpc_helper.events.events_manager import EventsManager
from grpc_helper.folders import Folders
from grpc_helper.logs.logs_manager import LogsManager
from grpc_helper.logs.logs_utils import add_rotating_handler, clean_rotating_handler
from grpc_helper.manager import RpcManager
from grpc_helper.methods import RpcSimpleMethod, RpcStreamingMethod
from grpc_helper.static_config import RpcStaticConfig
from grpc_helper.utils import is_streaming, trace_buffer

# Config file name
PROXY_FILE = "proxy.json"


# Persisted model keys
class ProxyModel:
    VERSION = "version"
    HOST = "host"
    PORT = "port"


class RpcServicer:
    """
    Generic servicer implementation, that:
     * logs method calls
     * checks api version
     * routes method to provided manager
    """

    def __init__(self, manager: RpcManager, stub: object, info: ServiceInfo, server: object):
        # Filter on manager methods
        logger = manager.logger if not info.is_proxy else logging.getLogger("RpcServer")
        logger.debug(f"Initializing RPC servicer for {info.name}")
        fake_stub = stub(insecure_channel("localhost:1"))
        for n in filter(lambda x: not x.startswith("_") and callable(getattr(manager, x)) and hasattr(fake_stub, x), dir(manager)):
            method = getattr(manager, n)
            sig = inspect.signature(method)
            return_type = sig.return_annotation

            # Only methods with declared return type + one input parameter (and we're not registering a proxy)
            if return_type != inspect._empty and len(sig.parameters) == 1 and not info.is_proxy:
                if return_type == Result:  # pragma: no cover
                    raise RpcException(f"Can't declare {n} rpc with Result as a return type; please use Return Status instead", ResultCode.ERROR_PARAM_INVALID)
                streaming = is_streaming(fake_stub, n)
                logger.debug(f" >> add method {n} (returns {return_type}){' [streaming]' if streaming else ''}")
                setattr(
                    self,
                    n,
                    RpcStreamingMethod(n, manager, None, return_type, info, server)
                    if streaming
                    else RpcSimpleMethod(n, manager, None, return_type, info, server),
                )
            # Otherwise, methods are coming from the parent stub
            else:
                if info.is_proxy:
                    # Register proxy method
                    streaming = is_streaming(fake_stub, n)
                    logger.debug(f" >> add proxy method {n}{' [streaming]' if streaming else ''}")
                    setattr(self, n, RpcStreamingMethod(n, None, stub, None, info, server) if streaming else RpcSimpleMethod(n, None, stub, None, info, server))
                else:
                    # Register stub method (not defined in the manager - will raise an exception on runtime)
                    logger.debug(f" >> add stub method {n}")
                    setattr(self, n, method)


@dataclass
class RpcServiceDescriptor:
    """
    Data class describing a service to be served by the RpcServer class

    Attributes:
        module: python module providing the service
        name: service name
        api_version: enum class holding the supported/current API versions
        manager: object instance to which delegating services requests; may be None for proxy services
        register_method: GRPC-generated method to be called for service registration on the server instance
        client_stub: GRPC-generated class holding the service client stub
        is_proxy: States if this service is a proxied one (to be forwarded to a remote RPC server)
    """

    module: ModuleType
    name: str
    api_version: EnumTypeWrapper
    manager: RpcManager
    register_method: Callable[[object, object], NoReturn]
    client_stub: TypeVar
    is_proxy: bool = False


class RpcServer(RpcServerServiceServicer, RpcManager):
    """
    Wrapper to GRPC api to setup an RPC server.

    Constructor arguments:
        port:
            TCP port to be listened by the RPC server (mandatory).
        descriptors:
            list of service RpcServiceDescriptor instances (mandatory).
        folders:
            Folders class instance, holding the different level folders (default: None).
        cli_config:
            dict of string:string config item defaults provided on the command line (default: None).
        static_items:
            list of Config/ConfigHolder instances for static config items (default: None).
        user_items:
            list of Config/ConfigHolder instances for user config items (default: None).
        with_events:
            states if this server has to serve the events service
        with_debug_signal:
            states if this server has to catch user signal to dump debug information
    """

    def __init__(
        self,
        port: int,
        descriptors: List[RpcServiceDescriptor],
        folders: Folders = None,
        cli_config: Dict[str, str] = None,
        static_items: List[Config] = None,
        user_items: List[Config] = None,
        with_events: bool = False,
        with_debug_signal: bool = True,
    ):
        RpcManager.__init__(self, PROXY_FILE)
        self.__port = port
        self.calls = []
        self.__shutdown_event = Event()

        # Prepare config manager
        # (by the way, add our own static items)
        config_m = ConfigManager(folders, cli_config, (static_items + [RpcStaticConfig]) if static_items is not None else [RpcStaticConfig], user_items)

        # Create server instance, disabling port reuse
        self.__server = server(futures.ThreadPoolExecutor(max_workers=RpcStaticConfig.MAX_WORKERS.int_val), options=[("grpc.so_reuseport", 0)])

        # Systematically add services:
        # - to handle server basic operations
        # - to handle remote user configuration update
        # - to handle remote loggers configuration update
        # - to handle events (if required)
        base_descriptors = [
            RpcServiceDescriptor(grpc_helper, "srv", ServerApiVersion, self, add_RpcServerServiceServicer_to_server, RpcServerServiceStub),
            RpcServiceDescriptor(grpc_helper, "config", ConfigApiVersion, config_m, add_ConfigServiceServicer_to_server, ConfigServiceStub),
            RpcServiceDescriptor(grpc_helper, "log", LoggerApiVersion, LogsManager(), add_LoggerServiceServicer_to_server, LoggerServiceStub),
        ]
        if with_events:
            base_descriptors.append(
                RpcServiceDescriptor(grpc_helper, "events", EventApiVersion, EventsManager(), add_EventServiceServicer_to_server, EventServiceStub)
            )
        self.descriptors = {d.name: d for d in base_descriptors}

        # Get servicers
        self.descriptors.update({d.name: d for d in descriptors})

        # Load persisted proxies model
        proxies_model = self._load_config(folders.workspace)

        # Prepare auto clients stubs map
        stubs_map = {}

        # Register everything
        self.__info = {}
        for descriptor in self.descriptors.values():
            # Prepare folders and logger (only for non-proxy)
            if not descriptor.is_proxy:
                descriptor.manager._init_folders_n_logger(folders, port)

            # Persisted model?
            persisted_model = proxies_model[descriptor.name] if descriptor.name in proxies_model else None

            # Build info for service
            versions = list(descriptor.api_version.values())
            versions.remove(0)
            current_version = max(versions)
            info = ServiceInfo(
                name=descriptor.name,
                version=persisted_model[ProxyModel.VERSION]
                if persisted_model is not None
                else f"{descriptor.module.__title__}:{descriptor.module.__version__}",
                current_api_version=current_version,
                supported_api_version=min(versions),
                is_proxy=descriptor.is_proxy,
                proxy_port=persisted_model[ProxyModel.PORT] if persisted_model is not None else None,
                proxy_host=persisted_model[ProxyModel.HOST] if persisted_model is not None else None,
            )
            self.logger.debug(f"Registering service in RPC server: {trace_buffer(info)}")

            # Remember info
            self.__info[info.name] = info

            # Register servicer in RPC server
            descriptor.register_method(RpcServicer(descriptor.manager, descriptor.client_stub, info, self), self.__server)

            # Contribute to auto-client stubs map
            stubs_map[descriptor.name] = (descriptor.client_stub, current_version)

        # Setup port and start
        self.logger.debug(f"Starting RPC server on port {self.__port}")
        try:
            self.__server.add_insecure_port(f"[::]:{self.__port}")
        except Exception as e:  # NOQA: B902
            msg = f"Failed to start RPC server on port {self.__port}: {e}"
            self.logger.error(msg)
            raise RpcException(msg, rc=ResultCode.ERROR_PORT_BUSY)
        self.__server.start()
        self.logger.debug(f"RPC server started on port {self.__port}")

        # Hook debug signal if required
        if with_debug_signal and os.name == "posix":
            signal.signal(signal.SIGUSR2, self.__dump_debug)

        # Load all managers (including client pointing to all served services)
        to_raise = None
        for descriptor in self.__real_descriptors:
            client = RpcClient(
                "localhost",
                self.__port,
                stubs_map,
                name=f"{descriptor.name}-client",
                timeout=RpcStaticConfig.CLIENT_TIMEOUT.float_val,
                logger=descriptor.manager.logger,
            )
            try:
                descriptor.manager._preload(client, folders)
            except Exception as e:
                self.logger.error(f"Error occurred during {descriptor.name} manager loading: {e}\n" + "".join(traceback.format_tb(e.__traceback__)))
                to_raise = e
                break

        # If something bad happened during managers loading, shutdown and raise exception
        if to_raise is not None:
            self.shutdown()
            raise to_raise

    @property
    def __real_descriptors(self):
        # Yield descriptors with a real manager registered (non-proxy ones)
        return filter(lambda d: not d.is_proxy, self.descriptors.values())

    def __dump_debug(self, signum, frame):
        """
        Dumps current threads + pending RPC requests
        """

        # Prepare debug output
        output = self._log_folder / f"RpcServerDump-{time.strftime('%Y%m%d%H%M%S')}.txt"
        with output.open("w") as f:
            # Dump threads
            faulthandler.dump_traceback(f, all_threads=True)

            # Dump pending calls
            f.write("\n\nPending RPC calls:\n")
            for call in list(self.calls):
                f.write(f"{call}\n")

    def shutdown(self, request: ShutdownRequest = None) -> ResultStatus:
        """
        Shuts down this RPC server instance
        """

        # Only if not shutdown yet
        if self.__server is not None:
            # Make sure we won't enter here twice
            terminating_server = self.__server
            self.__server = None

            # Stop server and wait for termination
            self.logger.debug(f"Shutting down RPC server on port {self.__port}; stop accepting new requests")
            terminating_server.stop(RpcStaticConfig.SHUTDOWN_GRACE.float_val)

            # Shutdown all managers
            self.logger.debug("Shutting down all managers")
            for descriptor in self.__real_descriptors:
                descriptor.manager._shutdown()

            # Is this an RPC call?
            if request is not None:
                # Yes --> delegate termination to its own thread, to return immediately from this request
                Thread(target=self.__finalize_shutdown, kwargs={"terminating_server": terminating_server, "request": request}, name="ShutdownThread").start()
            else:
                # Finalize shutdown immediately (internal call)
                self.__finalize_shutdown(terminating_server, None)

        # Always OK
        return ResultStatus()

    def __finalize_shutdown(self, terminating_server: Server, request: ShutdownRequest):
        # Wait for all pending requests to be terminated
        self.logger.debug("Waiting to terminate all requests")
        terminating_server.wait_for_termination()
        self.logger.debug(f"RPC server shut down on port {self.__port}")

        # Need to wait before real shutdown?
        # This may be useful to avoid being restarted by an orchestration manager (e.g. Docker Swarm), typically when doing a graceful shutdown before upgrade
        if request is not None and request.timeout >= 0:
            timeout = request.timeout if request.timeout > 0 else RpcStaticConfig.SHUTDOWN_TIMEOUT.int_val
            self.logger.warning(f"!!! Will shutdown in {timeout}s !!!")
            time.sleep(timeout)

        # Hack auto client to remove timeout
        self.client.srv.info.timeout = None

        # Just make sure that client calls are not working anymore with current instance
        # (Sometimes, it appears that the internal implementation is a bit lazy to close...)
        self.logger.debug("Trying a last client call to make sure server socket is closed (following ERROR is normal)")
        while True:
            try:
                # Try a client call
                self.client.srv.info(Filter())

                # Shouldn't get here; if so, wait a bit and retry
                time.sleep(0.2)  # pragma: no cover
            except Exception:
                # Ok, client is closed
                break

        # Removing all rotating loggers
        for descriptor in self.__real_descriptors:
            clean_rotating_handler(descriptor.manager.logger)

        # Remove rotating handler for current + root loggers
        clean_rotating_handler(logging.getLogger())
        self.__shutdown_event.set()

    def info(self, request: Filter) -> MultiServiceInfo:
        # Verify input service names
        if len(request.names):
            services = self.__check_service_names(request)
        else:
            services = self.__info.values()

        # Just build message from stored info
        return MultiServiceInfo(items=services)

    def _init_folders_n_logger(self, folders: Folders, port: int):
        # Override to process root logger as well
        super()._init_folders_n_logger(folders, port)
        add_rotating_handler(self._log_folder, logging.getLogger())

    @property
    def is_running(self) -> bool:
        # Just check is server is still running
        return not self.__shutdown_event.is_set()

    def __check_service_names(self, request: Union[ProxyRegisterRequest, Filter]) -> List[ServiceInfo]:
        if any(n not in self.__info for n in request.names):
            raise RpcException("At least one of the required service names is unknown", ResultCode.ERROR_ITEM_UNKNOWN)
        return list(map(lambda n: self.__info[n], request.names))

    def __check_proxy_names(self, request: Union[ProxyRegisterRequest, Filter]) -> List[ServiceInfo]:
        # Verify input proxy names
        if len(request.names) == 0 or any(len(x) == 0 for x in request.names):
            raise RpcException("Missing input name in request", ResultCode.ERROR_PARAM_MISSING)
        services = self.__check_service_names(request)
        if any(not srv.is_proxy for srv in services):
            raise RpcException("At least one of the required services is not a proxy", ResultCode.ERROR_PARAM_INVALID)
        return services

    def __persist_proxies(self):
        # Save all configured proxies
        model = {}
        for name, srv_info in self.__info.items():
            if srv_info.is_proxy and srv_info.proxy_port > 0:
                model[name] = {ProxyModel.HOST: srv_info.proxy_host, ProxyModel.PORT: srv_info.proxy_port, ProxyModel.VERSION: srv_info.version}
        self._save_config(model)

    @property
    def __can_send_events(self) -> bool:
        # Events manager (and client) is instantiated
        return hasattr(self.client, "events")

    def proxy_register(self, request: ProxyRegisterRequest) -> ResultStatus:
        # Verify input params
        services = self.__check_proxy_names(request)
        if len(request.version) == 0 or request.port == 0:
            raise RpcException("Missing input parameter in request (one of version or port)", ResultCode.ERROR_PARAM_MISSING)

        # Ready to update service proxy info
        with self.lock:
            for srv in services:
                # Update info
                srv.version = request.version
                srv.proxy_port = request.port
                srv.proxy_host = request.host

            # Persist proxy info
            self.__persist_proxies()

        # Send proxy registration event
        if self.__can_send_events:  # pragma: no branch
            self.client.events.send(
                RpcEvent(
                    name="RPC_PROXY_REGISTER",
                    properties=[
                        EventProperty(name="names", value=",".join(request.names)),
                        EventProperty(name="version", value=request.version),
                        EventProperty(name="port", value=str(request.port)),
                        EventProperty(name="host", value=request.host),
                    ],
                )
            )

        return ResultStatus()

    def proxy_forget(self, request: Filter) -> ResultStatus:
        # Verify input params
        services = self.__check_proxy_names(request)

        # Ready to update service proxy info
        with self.lock:
            for srv in services:
                # Update info
                srv.proxy_port = 0
                srv.proxy_host = ""

            # Persist proxy info
            self.__persist_proxies()

        # Send proxy forget event
        if self.__can_send_events:  # pragma: no branch
            self.client.events.send(RpcEvent(name="RPC_PROXY_FORGET", properties=[EventProperty(name="names", value=",".join(request.names))]))

        return ResultStatus()

    def wait_shutdown(self):
        """
        Waits until server is shutdown
        """

        # Simply wait for shutdown event
        self.__shutdown_event.wait()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_helper/api/logger.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grpc_helper.api import common_pb2 as grpc__helper_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grpc_helper/api/logger.proto",
    package="",
    syntax="proto3",
    serialized_options=b"\n\017grpc_helper.apiP\001",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cgrpc_helper/api/logger.proto\x1a\x1cgrpc_helper/api/common.proto"J\n\x0cLoggerConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x02 \x01(\x08\x12\x1b\n\x05level\x18\x03 \x01(\x0e\x32\x0c.LoggerLevel"@\n\x0cLoggerStatus\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result\x12\x1c\n\x05items\x18\x02 \x03(\x0b\x32\r.LoggerConfig",\n\x0cLoggerUpdate\x12\x1c\n\x05items\x18\x01 \x03(\x0b\x32\r.LoggerConfig*`\n\x10LoggerApiVersion\x12\x16\n\x12LOGGER_API_UNKNOWN\x10\x00\x12\x18\n\x14LOGGER_API_SUPPORTED\x10\x01\x12\x16\n\x12LOGGER_API_CURRENT\x10\x01\x1a\x02\x10\x01*m\n\x0bLoggerLevel\x12\x0f\n\x0bLVL_UNKNOWN\x10\x00\x12\r\n\tLVL_DEBUG\x10\n\x12\x0c\n\x08LVL_INFO\x10\x14\x12\x0f\n\x0bLVL_WARNING\x10\x1e\x12\r\n\tLVL_ERROR\x10(\x12\x10\n\x0cLVL_CRITICAL\x10\x32\x32t\n\rLoggerService\x12\x1d\n\x03get\x12\x07.Filter\x1a\r.LoggerStatus\x12\x1f\n\x05reset\x12\x07.Filter\x1a\r.LoggerStatus\x12#\n\x03set\x12\r.LoggerUpdate\x1a\r.LoggerStatusB\x13\n\x0fgrpc_helper.apiP\x01\x62\x06proto3',
    dependencies=[
        grpc__helper_dot_api_dot_common__pb2.DESCRIPTOR,
    ],
)

_LOGGERAPIVERSION = _descriptor.EnumDescriptor(
    name="LoggerApiVersion",
    full_name="LoggerApiVersion",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="LOGGER_API_UNKNOWN", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="LOGGER_API_SUPPORTED", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="LOGGER_API_CURRENT", index=2, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=b"\020\001",
    serialized_start=250,
    serialized_end=346,
)
_sym_db.RegisterEnumDescriptor(_LOGGERAPIVERSION)

LoggerApiVersion = enum_type_wrapper.EnumTypeWrapper(_LOGGERAPIVERSION)
_LOGGERLEVEL = _descriptor.EnumDescriptor(
    name="LoggerLevel",
    full_name="LoggerLevel",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(name="LVL_UNKNOWN", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(name="LVL_DEBUG", index=1, number=10, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(name="LVL_INFO", index=2, number=20, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name="LVL_WARNING", index=3, number=30, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(name="LVL_ERROR", index=4, number=40, serialized_options=None, type=None, create_key=_descriptor._internal_create_key),
        _descriptor.EnumValueDescriptor(
            name="LVL_CRITICAL", index=5, number=50, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=348,
    serialized_end=457,
)
_sym_db.RegisterEnumDescriptor(_LOGGERLEVEL)

LoggerLevel = enum_type_wrapper.EnumTypeWrapper(_LOGGERLEVEL)
LOGGER_API_UNKNOWN = 0
LOGGER_API_SUPPORTED = 1
LOGGER_API_CURRENT = 1
LVL_UNKNOWN = 0
LVL_DEBUG = 10
LVL_INFO = 20
LVL_WARNING = 30
LVL_ERROR = 40
LVL_CRITICAL = 50


_LOGGERCONFIG = _descriptor.Descriptor(
    name="LoggerConfig",
    full_name="LoggerConfig",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="LoggerConfig.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="enabled",
            full_name="LoggerConfig.enabled",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="level",
            full_name="LoggerConfig.level",
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=62,
    serialized_end=136,
)


_LOGGERSTATUS = _descriptor.Descriptor(
    name="LoggerStatus",
    full_name="LoggerStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="LoggerStatus.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="items",
            full_name="LoggerStatus.items",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=138,
    serialized_end=202,
)


_LOGGERUPDATE = _descriptor.Descriptor(
    name="LoggerUpdate",
    full_name="LoggerUpdate",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="items",
            full_name="LoggerUpdate.items",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=204,
    serialized_end=248,
)

_LOGGERCONFIG.fields_by_name["level"].enum_type = _LOGGERLEVEL
_LOGGERSTATUS.fields_by_name["r"].message_type = grpc__helper_dot_api_dot_common__pb2._RESULT
_LOGGERSTATUS.fields_by_name["items"].message_type = _LOGGERCONFIG
_LOGGERUPDATE.fields_by_name["items"].message_type = _LOGGERCONFIG
DESCRIPTOR.message_types_by_name["LoggerConfig"] = _LOGGERCONFIG
DESCRIPTOR.message_types_by_name["LoggerStatus"] = _LOGGERSTATUS
DESCRIPTOR.message_types_by_name["LoggerUpdate"] = _LOGGERUPDATE
DESCRIPTOR.enum_types_by_name["LoggerApiVersion"] = _LOGGERAPIVERSION
DESCRIPTOR.enum_types_by_name["LoggerLevel"] = _LOGGERLEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LoggerConfig = _reflection.GeneratedProtocolMessageType(
    "LoggerConfig",
    (_message.Message,),
    {
        "DESCRIPTOR": _LOGGERCONFIG,
        "__module__": "grpc_helper.api.logger_pb2"
        # @@protoc_insertion_point(class_scope:LoggerConfig)
    },
)
_sym_db.RegisterMessage(LoggerConfig)

LoggerStatus = _reflection.GeneratedProtocolMessageType(
    "LoggerStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _LOGGERSTATUS,
        "__module__": "grpc_helper.api.logger_pb2"
        # @@protoc_insertion_point(class_scope:LoggerStatus)
    },
)
_sym_db.RegisterMessage(LoggerStatus)

LoggerUpdate = _reflection.GeneratedProtocolMessageType(
    "LoggerUpdate",
    (_message.Message,),
    {
        "DESCRIPTOR": _LOGGERUPDATE,
        "__module__": "grpc_helper.api.logger_pb2"
        # @@protoc_insertion_point(class_scope:LoggerUpdate)
    },
)
_sym_db.RegisterMessage(LoggerUpdate)


DESCRIPTOR._options = None
_LOGGERAPIVERSION._options = None

_LOGGERSERVICE = _descriptor.ServiceDescriptor(
    name="LoggerService",
    full_name="LoggerService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=459,
    serialized_end=575,
    methods=[
        _descriptor.MethodDescriptor(
            name="get",
            full_name="LoggerService.get",
            index=0,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=_LOGGERSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="reset",
            full_name="LoggerService.reset",
            index=1,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=_LOGGERSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="set",
            full_name="LoggerService.set",
            index=2,
            containing_service=None,
            input_type=_LOGGERUPDATE,
            output_type=_LOGGERSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_LOGGERSERVICE)

DESCRIPTOR.services_by_name["LoggerService"] = _LOGGERSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_helper/api/config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grpc_helper.api import common_pb2 as grpc__helper_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grpc_helper/api/config.proto",
    package="",
    syntax="proto3",
    serialized_options=b"\n\017grpc_helper.apiP\001",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cgrpc_helper/api/config.proto\x1a\x1cgrpc_helper/api/common.proto"\x90\x01\n\nConfigItem\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x15\n\rdefault_value\x18\x04 \x01(\t\x12\x14\n\x0c\x63\x61n_be_empty\x18\x05 \x01(\x08\x12#\n\tvalidator\x18\x06 \x01(\x0e\x32\x10.ConfigValidator">\n\x0c\x43onfigStatus\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result\x12\x1a\n\x05items\x18\x02 \x03(\x0b\x32\x0b.ConfigItem"/\n\x10\x43onfigItemUpdate\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"H\n\x0c\x43onfigUpdate\x12 \n\x05items\x18\x01 \x03(\x0b\x32\x11.ConfigItemUpdate\x12\x16\n\x0eignore_unknown\x18\x02 \x01(\x08*\\\n\x10\x43onfigApiVersion\x12\x16\n\x12\x43ONFIG_API_UNKNOWN\x10\x00\x12\x18\n\x14\x43ONFIG_API_SUPPORTED\x10\x01\x12\x16\n\x12\x43ONFIG_API_CURRENT\x10\x02*\xa7\x01\n\x0f\x43onfigValidator\x12\x17\n\x13\x43ONFIG_VALID_STRING\x10\x00\x12\x17\n\x13\x43ONFIG_VALID_CUSTOM\x10\x01\x12\x14\n\x10\x43ONFIG_VALID_INT\x10\x02\x12\x18\n\x14\x43ONFIG_VALID_POS_INT\x10\x03\x12\x16\n\x12\x43ONFIG_VALID_FLOAT\x10\x04\x12\x1a\n\x16\x43ONFIG_VALID_POS_FLOAT\x10\x05\x32t\n\rConfigService\x12\x1d\n\x03get\x12\x07.Filter\x1a\r.ConfigStatus\x12\x1f\n\x05reset\x12\x07.Filter\x1a\r.ConfigStatus\x12#\n\x03set\x12\r.ConfigUpdate\x1a\r.ConfigStatusB\x13\n\x0fgrpc_helper.apiP\x01\x62\x06proto3',
    dependencies=[
        grpc__helper_dot_api_dot_common__pb2.DESCRIPTOR,
    ],
)

_CONFIGAPIVERSION = _descriptor.EnumDescriptor(
    name="ConfigApiVersion",
    full_name="ConfigApiVersion",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="CONFIG_API_UNKNOWN", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_API_SUPPORTED", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_API_CURRENT", index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=396,
    serialized_end=488,
)
_sym_db.RegisterEnumDescriptor(_CONFIGAPIVERSION)

ConfigApiVersion = enum_type_wrapper.EnumTypeWrapper(_CONFIGAPIVERSION)
_CONFIGVALIDATOR = _descriptor.EnumDescriptor(
    name="ConfigValidator",
    full_name="ConfigValidator",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_STRING", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_CUSTOM", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_INT", index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_POS_INT", index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_FLOAT", index=4, number=4, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="CONFIG_VALID_POS_FLOAT", index=5, number=5, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=491,
    serialized_end=658,
)
_sym_db.RegisterEnumDescriptor(_CONFIGVALIDATOR)

ConfigValidator = enum_type_wrapper.EnumTypeWrapper(_CONFIGVALIDATOR)
CONFIG_API_UNKNOWN = 0
CONFIG_API_SUPPORTED = 1
CONFIG_API_CURRENT = 2
CONFIG_VALID_STRING = 0
CONFIG_VALID_CUSTOM = 1
CONFIG_VALID_INT = 2
CONFIG_VALID_POS_INT = 3
CONFIG_VALID_FLOAT = 4
CONFIG_VALID_POS_FLOAT = 5


_CONFIGITEM = _descriptor.Descriptor(
    name="ConfigItem",
    full_name="ConfigItem",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ConfigItem.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="ConfigItem.description",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="ConfigItem.value",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="default_value",
            full_name="ConfigItem.default_value",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="can_be_empty",
            full_name="ConfigItem.can_be_empty",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="validator",
            full_name="ConfigItem.validator",
            index=5,
            number=6,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=63,
    serialized_end=207,
)


_CONFIGSTATUS = _descriptor.Descriptor(
    name="ConfigStatus",
    full_name="ConfigStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="ConfigStatus.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="items",
            full_name="ConfigStatus.items",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=209,
    serialized_end=271,
)


_CONFIGITEMUPDATE = _descriptor.Descriptor(
    name="ConfigItemUpdate",
    full_name="ConfigItemUpdate",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ConfigItemUpdate.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="ConfigItemUpdate.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=273,
    serialized_end=320,
)


_CONFIGUPDATE = _descriptor.Descriptor(
    name="ConfigUpdate",
    full_name="ConfigUpdate",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="items",
            full_name="ConfigUpdate.items",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ignore_unknown",
            full_name="ConfigUpdate.ignore_unknown",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=322,
    serialized_end=394,
)

_CONFIGITEM.fields_by_name["validator"].enum_type = _CONFIGVALIDATOR
_CONFIGSTATUS.fields_by_name["r"].message_type = grpc__helper_dot_api_dot_common__pb2._RESULT
_CONFIGSTATUS.fields_by_name["items"].message_type = _CONFIGITEM
_CONFIGUPDATE.fields_by_name["items"].message_type = _CONFIGITEMUPDATE
DESCRIPTOR.message_types_by_name["ConfigItem"] = _CONFIGITEM
DESCRIPTOR.message_types_by_name["ConfigStatus"] = _CONFIGSTATUS
DESCRIPTOR.message_types_by_name["ConfigItemUpdate"] = _CONFIGITEMUPDATE
DESCRIPTOR.message_types_by_name["ConfigUpdate"] = _CONFIGUPDATE
DESCRIPTOR.enum_types_by_name["ConfigApiVersion"] = _CONFIGAPIVERSION
DESCRIPTOR.enum_types_by_name["ConfigValidator"] = _CONFIGVALIDATOR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ConfigItem = _reflection.GeneratedProtocolMessageType(
    "ConfigItem",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONFIGITEM,
        "__module__": "grpc_helper.api.config_pb2"
        # @@protoc_insertion_point(class_scope:ConfigItem)
    },
)
_sym_db.RegisterMessage(ConfigItem)

ConfigStatus = _reflection.GeneratedProtocolMessageType(
    "ConfigStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONFIGSTATUS,
        "__module__": "grpc_helper.api.config_pb2"
        # @@protoc_insertion_point(class_scope:ConfigStatus)
    },
)
_sym_db.RegisterMessage(ConfigStatus)

ConfigItemUpdate = _reflection.GeneratedProtocolMessageType(
    "ConfigItemUpdate",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONFIGITEMUPDATE,
        "__module__": "grpc_helper.api.config_pb2"
        # @@protoc_insertion_point(class_scope:ConfigItemUpdate)
    },
)
_sym_db.RegisterMessage(ConfigItemUpdate)

ConfigUpdate = _reflection.GeneratedProtocolMessageType(
    "ConfigUpdate",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONFIGUPDATE,
        "__module__": "grpc_helper.api.config_pb2"
        # @@protoc_insertion_point(class_scope:ConfigUpdate)
    },
)
_sym_db.RegisterMessage(ConfigUpdate)


DESCRIPTOR._options = None

_CONFIGSERVICE = _descriptor.ServiceDescriptor(
    name="ConfigService",
    full_name="ConfigService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=660,
    serialized_end=776,
    methods=[
        _descriptor.MethodDescriptor(
            name="get",
            full_name="ConfigService.get",
            index=0,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=_CONFIGSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="reset",
            full_name="ConfigService.reset",
            index=1,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=_CONFIGSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="set",
            full_name="ConfigService.set",
            index=2,
            containing_service=None,
            input_type=_CONFIGUPDATE,
            output_type=_CONFIGSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_CONFIGSERVICE)

DESCRIPTOR.services_by_name["ConfigService"] = _CONFIGSERVICE

# @@protoc_insertion_point(module_scope)



import re
from typing import Callable, List

from grpc_helper.api import ConfigItem, ConfigValidator, ResultCode
from grpc_helper.errors import RpcException

# Pattern for config item name
NAME_PATTERN = re.compile("[a-z][a-z0-9-]*")


# Integer validation
def validate_int(name: str, value: str):
    try:
        return int(value)
    except ValueError:
        raise RpcException(f"Invalid int value for config item {name}: {value}", rc=ResultCode.ERROR_PARAM_INVALID)


# Positive number validation
def validate_pos(name: str, value: str, type_validator: Callable):
    if type_validator(name, value) <= 0:
        raise RpcException(f"Expected strictly positive value for config item {name} but got {value}", rc=ResultCode.ERROR_PARAM_INVALID)


# Positive integer validation
def validate_pos_int(name: str, value: str):
    validate_pos(name, value, validate_int)


# Float validation
def validate_float(name: str, value: str):
    try:
        return float(value)
    except ValueError:
        raise RpcException(f"Invalid float value for config item {name}: {value}", rc=ResultCode.ERROR_PARAM_INVALID)


# Float validation
def validate_pos_float(name: str, value: str):
    validate_pos(name, value, validate_float)


# Built-Validators
VALIDATORS = {
    ConfigValidator.CONFIG_VALID_STRING: lambda _n, _v: None,
    ConfigValidator.CONFIG_VALID_INT: validate_int,
    ConfigValidator.CONFIG_VALID_POS_INT: validate_pos_int,
    ConfigValidator.CONFIG_VALID_FLOAT: validate_float,
    ConfigValidator.CONFIG_VALID_POS_FLOAT: validate_pos_float,
}


class Config:
    """
    API for configuration items.

    Constructor arguments are the same than the ConfigItem ones, with an additional custom_validator one,
    allowing to define a custom validation method.
    This method must takes two arguments:
    * a "name" string: may be useful to raise meaningful validation errors
    * a "value" string: to be validated. The method shall raise an RpcException if the value is invalid.
    """

    def __init__(self, custom_validator: Callable = None, **kwargs):
        # Create item with kwargs
        self.item = ConfigItem(**kwargs)

        # Preserve hard-coded default (mainly for unit tests, which are re-using instances with modified defaults)
        self.hard_coded_default_value = self.item.default_value

        # Validate name
        m = NAME_PATTERN.match(self.item.name)
        if m is None:
            raise RpcException(f"Invalid config item name: {self.item.name}", ResultCode.ERROR_PARAM_INVALID)

        # Validate validator
        if self.item.validator == ConfigValidator.CONFIG_VALID_CUSTOM and custom_validator is None:
            raise RpcException(f"Missing custom validator for config item: {self.item.name}", ResultCode.ERROR_PARAM_MISSING)

        # Validation operation delegated to configured implementation
        self.__validate = custom_validator if self.item.validator == ConfigValidator.CONFIG_VALID_CUSTOM else VALIDATORS[self.item.validator]

    @property
    def name(self) -> str:
        return self.item.name

    @property
    def default_value(self) -> str:
        return self.item.default_value

    @property
    def str_val(self) -> str:
        return self.item.value

    @property
    def int_val(self) -> int:
        return int(self.str_val)

    @property
    def float_val(self) -> float:
        return float(self.str_val)

    def reset(self):
        # Reset item value to its default one
        self.update(self.item.default_value)

    def update(self, value: str):
        # Validate item before update
        self.validate(self.name, value)

        # Update item value
        self.item.value = value

    def validate(self, name: str, value: str):
        # Can be empty?
        if value == "":
            if not self.item.can_be_empty:
                raise RpcException(f"Empty value provided for item {name} (not supported)", rc=ResultCode.ERROR_PARAM_MISSING)
        else:
            # Delegate to validator
            self.__validate(name, value)


class ConfigHolder:
    """
    A base class from which configuration items holders may inherit
    """

    @classmethod
    def all_config_items(cls) -> List[Config]:
        return list(filter(lambda x: isinstance(x, Config), cls.__dict__.values()))



from grpc_helper.api import ConfigValidator, ResultCode
from grpc_helper.config.cfg_item import Config, ConfigHolder
from grpc_helper.errors import RpcException

# Allowed interval units
INTERVAL_UNITS = ["S", "M", "H", "D", "MIDNIGHT"] + list(map(lambda x: f"W{x}", range(7)))


# Interval unit validation
def validate_interval_unit(name: str, value: str):
    if value.upper() not in INTERVAL_UNITS:
        raise RpcException(f"Invalid interval unit for {name}: {value}", rc=ResultCode.ERROR_PARAM_INVALID)


# Internal config for RPC servers
class RpcStaticConfig(ConfigHolder):
    MAX_WORKERS = Config(
        name="rpc-max-workers", description="Maximum parallel RPC worker threads", default_value="30", validator=ConfigValidator.CONFIG_VALID_POS_INT
    )
    SHUTDOWN_GRACE = Config(
        name="rpc-shutdown-grace",
        description="Grace period for pending calls to be terminated on shutdown (seconds)",
        default_value="30",
        validator=ConfigValidator.CONFIG_VALID_POS_FLOAT,
    )
    SHUTDOWN_TIMEOUT = Config(
        name="rpc-shutdown-timeout",
        description="Final timeout before real shutdown (i.e. end of process; seconds)",
        default_value="60",
        validator=ConfigValidator.CONFIG_VALID_POS_FLOAT,
    )
    LOGS_FOLDER = Config(name="rpc-logs-folder", description="Workspace relative folder where to store rolling logs", default_value="logs")
    LOGS_BACKUP = Config(
        name="rpc-logs-backup",
        description="Backup log files to be persisted for each manager on rollover",
        default_value="10",
        validator=ConfigValidator.CONFIG_VALID_INT,
    )
    LOGS_ROLLOVER_INTERVAL_UNIT = Config(
        name="rpc-logs-interval-unit",
        description="Rollover interval unit (see TimedRotatingFileHandler documentation)",
        default_value="H",
        validator=ConfigValidator.CONFIG_VALID_CUSTOM,
        custom_validator=validate_interval_unit,
    )
    LOGS_ROLLOVER_INTERVAL = Config(
        name="rpc-logs-interval",
        description="Rollover interval (see TimedRotatingFileHandler documentation)",
        default_value="1",
        validator=ConfigValidator.CONFIG_VALID_POS_INT,
    )
    MAIN_HOST = Config(name="rpc-main-host", description="Main RPC server host (to be used by proxied services)", default_value="localhost")
    MAIN_PORT = Config(
        name="rpc-main-port",
        description="Main RPC server port (to be used by proxied services)",
        default_value="54321",
        validator=ConfigValidator.CONFIG_VALID_POS_INT,
    )
    CLIENT_TIMEOUT = Config(
        name="rpc-client-timeout",
        description="Timeout for RPC client when server is unreachable or proxy not registered yet (seconds)",
        default_value="60",
        validator=ConfigValidator.CONFIG_VALID_POS_FLOAT,
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_helper/api/server.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grpc_helper.api import common_pb2 as grpc__helper_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grpc_helper/api/server.proto",
    package="",
    syntax="proto3",
    serialized_options=b"\n\017grpc_helper.apiP\001",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cgrpc_helper/api/server.proto\x1a\x1cgrpc_helper/api/common.proto"\xa2\x01\n\x0bServiceInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x1b\n\x13\x63urrent_api_version\x18\x03 \x01(\r\x12\x1d\n\x15supported_api_version\x18\x04 \x01(\r\x12\x10\n\x08is_proxy\x18\x05 \x01(\x08\x12\x12\n\nproxy_host\x18\x06 \x01(\t\x12\x12\n\nproxy_port\x18\x07 \x01(\r"C\n\x10MultiServiceInfo\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result\x12\x1b\n\x05items\x18\x02 \x03(\x0b\x32\x0c.ServiceInfo""\n\x0fShutdownRequest\x12\x0f\n\x07timeout\x18\x01 \x01(\x05"R\n\x14ProxyRegisterRequest\x12\r\n\x05names\x18\x01 \x03(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x0c\n\x04host\x18\x03 \x01(\t\x12\x0c\n\x04port\x18\x04 \x01(\r*`\n\x10ServerApiVersion\x12\x16\n\x12SERVER_API_UNKNOWN\x10\x00\x12\x18\n\x14SERVER_API_SUPPORTED\x10\x01\x12\x16\n\x12SERVER_API_CURRENT\x10\x01\x1a\x02\x10\x01\x32\xc3\x01\n\x10RpcServerService\x12"\n\x04info\x12\x07.Filter\x1a\x11.MultiServiceInfo\x12+\n\x08shutdown\x12\x10.ShutdownRequest\x1a\r.ResultStatus\x12\x36\n\x0eproxy_register\x12\x15.ProxyRegisterRequest\x1a\r.ResultStatus\x12&\n\x0cproxy_forget\x12\x07.Filter\x1a\r.ResultStatusB\x13\n\x0fgrpc_helper.apiP\x01\x62\x06proto3',
    dependencies=[
        grpc__helper_dot_api_dot_common__pb2.DESCRIPTOR,
    ],
)

_SERVERAPIVERSION = _descriptor.EnumDescriptor(
    name="ServerApiVersion",
    full_name="ServerApiVersion",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="SERVER_API_UNKNOWN", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="SERVER_API_SUPPORTED", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="SERVER_API_CURRENT", index=2, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=b"\020\001",
    serialized_start=416,
    serialized_end=512,
)
_sym_db.RegisterEnumDescriptor(_SERVERAPIVERSION)

ServerApiVersion = enum_type_wrapper.EnumTypeWrapper(_SERVERAPIVERSION)
SERVER_API_UNKNOWN = 0
SERVER_API_SUPPORTED = 1
SERVER_API_CURRENT = 1


_SERVICEINFO = _descriptor.Descriptor(
    name="ServiceInfo",
    full_name="ServiceInfo",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ServiceInfo.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="version",
            full_name="ServiceInfo.version",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="current_api_version",
            full_name="ServiceInfo.current_api_version",
            index=2,
            number=3,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="supported_api_version",
            full_name="ServiceInfo.supported_api_version",
            index=3,
            number=4,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="is_proxy",
            full_name="ServiceInfo.is_proxy",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="proxy_host",
            full_name="ServiceInfo.proxy_host",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="proxy_port",
            full_name="ServiceInfo.proxy_port",
            index=6,
            number=7,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=63,
    serialized_end=225,
)


_MULTISERVICEINFO = _descriptor.Descriptor(
    name="MultiServiceInfo",
    full_name="MultiServiceInfo",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="MultiServiceInfo.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="items",
            full_name="MultiServiceInfo.items",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=227,
    serialized_end=294,
)


_SHUTDOWNREQUEST = _descriptor.Descriptor(
    name="ShutdownRequest",
    full_name="ShutdownRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="timeout",
            full_name="ShutdownRequest.timeout",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=296,
    serialized_end=330,
)


_PROXYREGISTERREQUEST = _descriptor.Descriptor(
    name="ProxyRegisterRequest",
    full_name="ProxyRegisterRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="names",
            full_name="ProxyRegisterRequest.names",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="version",
            full_name="ProxyRegisterRequest.version",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="host",
            full_name="ProxyRegisterRequest.host",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="port",
            full_name="ProxyRegisterRequest.port",
            index=3,
            number=4,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=332,
    serialized_end=414,
)

_MULTISERVICEINFO.fields_by_name["r"].message_type = grpc__helper_dot_api_dot_common__pb2._RESULT
_MULTISERVICEINFO.fields_by_name["items"].message_type = _SERVICEINFO
DESCRIPTOR.message_types_by_name["ServiceInfo"] = _SERVICEINFO
DESCRIPTOR.message_types_by_name["MultiServiceInfo"] = _MULTISERVICEINFO
DESCRIPTOR.message_types_by_name["ShutdownRequest"] = _SHUTDOWNREQUEST
DESCRIPTOR.message_types_by_name["ProxyRegisterRequest"] = _PROXYREGISTERREQUEST
DESCRIPTOR.enum_types_by_name["ServerApiVersion"] = _SERVERAPIVERSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ServiceInfo = _reflection.GeneratedProtocolMessageType(
    "ServiceInfo",
    (_message.Message,),
    {
        "DESCRIPTOR": _SERVICEINFO,
        "__module__": "grpc_helper.api.server_pb2"
        # @@protoc_insertion_point(class_scope:ServiceInfo)
    },
)
_sym_db.RegisterMessage(ServiceInfo)

MultiServiceInfo = _reflection.GeneratedProtocolMessageType(
    "MultiServiceInfo",
    (_message.Message,),
    {
        "DESCRIPTOR": _MULTISERVICEINFO,
        "__module__": "grpc_helper.api.server_pb2"
        # @@protoc_insertion_point(class_scope:MultiServiceInfo)
    },
)
_sym_db.RegisterMessage(MultiServiceInfo)

ShutdownRequest = _reflection.GeneratedProtocolMessageType(
    "ShutdownRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _SHUTDOWNREQUEST,
        "__module__": "grpc_helper.api.server_pb2"
        # @@protoc_insertion_point(class_scope:ShutdownRequest)
    },
)
_sym_db.RegisterMessage(ShutdownRequest)

ProxyRegisterRequest = _reflection.GeneratedProtocolMessageType(
    "ProxyRegisterRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _PROXYREGISTERREQUEST,
        "__module__": "grpc_helper.api.server_pb2"
        # @@protoc_insertion_point(class_scope:ProxyRegisterRequest)
    },
)
_sym_db.RegisterMessage(ProxyRegisterRequest)


DESCRIPTOR._options = None
_SERVERAPIVERSION._options = None

_RPCSERVERSERVICE = _descriptor.ServiceDescriptor(
    name="RpcServerService",
    full_name="RpcServerService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=515,
    serialized_end=710,
    methods=[
        _descriptor.MethodDescriptor(
            name="info",
            full_name="RpcServerService.info",
            index=0,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=_MULTISERVICEINFO,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="shutdown",
            full_name="RpcServerService.shutdown",
            index=1,
            containing_service=None,
            input_type=_SHUTDOWNREQUEST,
            output_type=grpc__helper_dot_api_dot_common__pb2._RESULTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="proxy_register",
            full_name="RpcServerService.proxy_register",
            index=2,
            containing_service=None,
            input_type=_PROXYREGISTERREQUEST,
            output_type=grpc__helper_dot_api_dot_common__pb2._RESULTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="proxy_forget",
            full_name="RpcServerService.proxy_forget",
            index=3,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._FILTER,
            output_type=grpc__helper_dot_api_dot_common__pb2._RESULTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_RPCSERVERSERVICE)

DESCRIPTOR.services_by_name["RpcServerService"] = _RPCSERVERSERVICE

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from grpc_helper.api import common_pb2 as grpc__helper_dot_api_dot_common__pb2
from grpc_helper.api import events_pb2 as grpc__helper_dot_api_dot_events__pb2


class EventServiceStub(object):
    """Event service
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.listen = channel.unary_stream(
                '/EventService/listen',
                request_serializer=grpc__helper_dot_api_dot_common__pb2.Filter.SerializeToString,
                response_deserializer=grpc__helper_dot_api_dot_events__pb2.EventStatus.FromString,
                )
        self.interrupt = channel.unary_unary(
                '/EventService/interrupt',
                request_serializer=grpc__helper_dot_api_dot_events__pb2.EventInterrupt.SerializeToString,
                response_deserializer=grpc__helper_dot_api_dot_common__pb2.ResultStatus.FromString,
                )
        self.send = channel.unary_unary(
                '/EventService/send',
                request_serializer=grpc__helper_dot_api_dot_events__pb2.Event.SerializeToString,
                response_deserializer=grpc__helper_dot_api_dot_common__pb2.ResultStatus.FromString,
                )


class EventServiceServicer(object):
    """Event service
    """

    def listen(self, request, context):
        """Listen to events
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def interrupt(self, request, context):
        """Interrupt listening
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def send(self, request, context):
        """Send event
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EventServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'listen': grpc.unary_stream_rpc_method_handler(
                    servicer.listen,
                    request_deserializer=grpc__helper_dot_api_dot_common__pb2.Filter.FromString,
                    response_serializer=grpc__helper_dot_api_dot_events__pb2.EventStatus.SerializeToString,
            ),
            'interrupt': grpc.unary_unary_rpc_method_handler(
                    servicer.interrupt,
                    request_deserializer=grpc__helper_dot_api_dot_events__pb2.EventInterrupt.FromString,
                    response_serializer=grpc__helper_dot_api_dot_common__pb2.ResultStatus.SerializeToString,
            ),
            'send': grpc.unary_unary_rpc_method_handler(
                    servicer.send,
                    request_deserializer=grpc__helper_dot_api_dot_events__pb2.Event.FromString,
                    response_serializer=grpc__helper_dot_api_dot_common__pb2.ResultStatus.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'EventService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class EventService(object):
    """Event service
    """

    @staticmethod
    def listen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/EventService/listen',
            grpc__helper_dot_api_dot_common__pb2.Filter.SerializeToString,
            grpc__helper_dot_api_dot_events__pb2.EventStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def interrupt(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/EventService/interrupt',
            grpc__helper_dot_api_dot_events__pb2.EventInterrupt.SerializeToString,
            grpc__helper_dot_api_dot_common__pb2.ResultStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def send(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/EventService/send',
            grpc__helper_dot_api_dot_events__pb2.Event.SerializeToString,
            grpc__helper_dot_api_dot_common__pb2.ResultStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from queue import Queue

from grpc_helper.api import Event, EventInterrupt, EventStatus, Filter, ResultCode, ResultStatus
from grpc_helper.api.common_pb2 import Result
from grpc_helper.api.events_pb2_grpc import EventServiceServicer
from grpc_helper.errors import RpcException
from grpc_helper.manager import RpcManager


class EventsManager(EventServiceServicer, RpcManager):
    """
    Events manager, and implementing the EventService API
    """

    def __init__(self):
        # Super call
        super().__init__()

        # Some setup
        self.__queues = {}

    def _shutdown(self):
        # On shutdown, interrupt all pending queues
        with self.lock:
            for q in self.__queues.values():
                q.put(EventStatus(r=Result(msg="Service is shutdown", code=ResultCode.ERROR_STREAM_SHUTDOWN)), False)

    def __get_queue(self, index: int) -> Queue:
        with self.lock:
            return self.__queues[index] if index in self.__queues else None

    def interrupt(self, request: EventInterrupt) -> ResultStatus:
        # Known index?
        q = self.__get_queue(request.client_id)
        if q is None:
            raise RpcException(f"Unknown event listening ID: {request.client_id}", ResultCode.ERROR_ITEM_UNKNOWN)

        # Put "None" event (will wait until listen loop is over)
        q.put(None)
        return ResultStatus()

    def listen(self, request: Filter) -> EventStatus:
        # Validate filter
        for name in request.names:
            if len(name) == 0:
                raise RpcException("Empty name in filter request", ResultCode.ERROR_PARAM_MISSING)
            if " " in name:
                raise RpcException(f"Invalid name in filter request: {name}", ResultCode.ERROR_PARAM_INVALID)

        # Prepare a new queue
        with self.lock:
            # Prepare a new index (from scratch, to reuse released indexes)
            index = 1
            while index in self.__queues:
                index += 1

            # Create new queue
            q = Queue()
            self.__queues[index] = q

        # Yield at least a first status with client_id
        yield EventStatus(client_id=index)

        # Loop to flush received events
        while True:
            # Blocking read
            event = q.get()

            # End of loop?
            if event is None or isinstance(event, EventStatus):
                break

            # Is this a meaningful event?
            if len(request.names) == 0 or event.name in request.names:
                # Yield received events (if any)
                yield EventStatus(client_id=index, event=event)

        # Forget queue
        with self.lock:
            del self.__queues[index]

        # Last event to be yield (in case of shutdown)?
        if isinstance(event, EventStatus):
            yield event

    def send(self, request: Event) -> ResultStatus:
        # Just check event name is correct
        if len(request.name) == 0:
            raise RpcException("Empty event name", ResultCode.ERROR_PARAM_MISSING)
        if " " in request.name:
            raise RpcException(f"Invalid event name: {request.name}", ResultCode.ERROR_PARAM_INVALID)

        # Add event to all queues
        with self.lock:
            for q in self.__queues.values():
                q.put_nowait(request)

        return ResultStatus()
